import React, { useEffect, useState, useRef, forwardRef, useImperativeHandle } from 'react';
import { Button, Modal, List, Popconfirm, Form, Select, InputNumber } from 'antd';
import classNames from 'classnames';
import useBool from '@/utils/hooks/useBool';
import { getPackageDetailList, addPackageDetail, deletePackageDetail } from '@/services/package';
import { getDishList } from '@/services/dish';

const { Option } = Select;
const defaultKind = 2;

const AddDish = props => {
  const { onChange } = props;
  const [data, setData] = useState([]);
  const [form] = Form.useForm();

  const onFinish = (values) => {
    const it = data.filter(x => x.id === values.dishId)[0] || {};
    onChange({...it, ...values });
  };

  const handleSearch = (newValue) => {
    if (newValue) {
      getDishList({ name: newValue, pageNum: 1, pageSize: 100 }).then(res => {
        setData(res.records || []);
      });
    } else {
      setData([]);
    }
  };

  const options = data.map(d => <Option key={d.id} value={d.id}>{d.name}</Option>);

  const initList = () => {
    getDishList({ pageNum: 1, pageSize: 20 }).then(res => {
      setData(res.records || []);
    });
  }

  useEffect(() => {
    initList();
    form.setFieldsValue({ dishKind: 2, amount: 1 });
  }, [])

  return (
    <Form form={form} layout="vertical" onFinish={onFinish}>
      <Form.Item name="dishId" label="选择菜肴" rules={[{ required: true, message: '请选择菜肴' }]}>
        <Select
          allowClear
          showSearch
          placeholder='请选择菜肴, 可输入汉字搜索'
          defaultActiveFirstOption={false}
          showArrow={false}
          filterOption={false}
          onSearch={handleSearch}
          // onChange={handleChange}
          notFoundContent={null}
          onClear={initList}
        >
          {options}
        </Select>
      </Form.Item>
      <Form.Item name="dishKind" label="菜肴类型">
        <Select allowClear defaultValue={defaultKind}>
          <Select.Option value={1}>主食</Select.Option>
          <Select.Option value={2}>菜肴</Select.Option>
        </Select>
      </Form.Item>
      <Form.Item name="amount" label="菜肴数量" rules={[{ required: true, message: '请设置数量' }]}>
        <InputNumber style={{ width: '100%' }} min={1} />
      </Form.Item>
      <Form.Item>
        <Button type='primary' htmlType='sumbit'>确定</Button>
      </Form.Item>
    </Form>
  );
}

const kindDict = {
  '1': '主食',
  '2': '菜肴',
}

export default forwardRef((props, ref) => {
  const { current, setCurrent } = props;

  const packageId = (current || {}).id;

  const [loading, setLoading] = useState(false);
  const [open, setVisible, hide ] = useBool();
  const [list, setList] = useState([]);
  const [detail, setDetail] = useState({});
  const listRef = useRef();
  listRef.current = list;

  const onAdd = item => {
    setLoading(true);
    addPackageDetail({ ...item, packageId, dishId: item.id }).then(res => {
      setLoading(false);
      const newList = listRef.current.concat(res);
      setList(newList);
      hide();
    }).catch(() => {
      setLoading(false);
    });
  }

  const onDelete = item => {
    setLoading(true);
    deletePackageDetail({ packageId, dishId: item.id }).then(res => {
      setLoading(false);
      const newList = listRef.current.filter(x => x.id !== item.id);
      setList(newList);
    }).catch(() => {
      setLoading(false);
    });
  }

  useEffect(() => {
    if (!packageId) return;

    setLoading(true);
    getPackageDetailList({ packageId, pageNum: 1, pageSize: 500 }).then(res => {
      setLoading(false);
      setList(res.records || []);
      setDetail((res.records || [])[0] || {});
    }).catch(() => {
      setLoading(false);
    });

  }, [packageId]);

  useImperativeHandle(ref, () => ({
    new: () => setVisible(),
  }));

  return (
    <>
      <List
        style={{ minHeight: '300px' }}
        loading={loading}
        dataSource={list}
        renderItem={item => (
          <List.Item
            className={classNames({ active: detail.id === item.id })}
            onClick={() => setDetail(item)}
            actions={[
              <Popconfirm
                key="delete"
                title="确认删除?"
                onConfirm={() => onDelete(item)}
                >
                <a href="#">删除</a>
              </Popconfirm>
            ]}
          >
            {`【${kindDict[`${item.dishKind}`]}】${item.name} x ${item.amount}`}
          </List.Item>
        )}
      />
      <Modal
        title="添加菜肴"
        open={open}
        onCancel={hide}
        maskClosable={false}
        footer={null}
        destroyOnClose
      >
        <AddDish onChange={onAdd} />
      </Modal>
    </>
  );
})
